import 'package:osam/domain/event/event.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/domain/store/store.dart';
import 'package:osam/navigation/domain/navigation_state.dart';
import 'package:osam/presentation/presenter.dart';
import 'package:osam/util/state_stream.dart';

import 'navigation_presenter_interface.dart';

abstract class NavigationPresenter<S extends Store<BaseState>, T> extends Presenter<S>
    implements INavigation<T> {
  NavigationState<T> get state;

  @override
  void pop() => store.dispatchEvent(event: Event<NavigationState<T>>(reducer: (_) => state..pop()));

  @override
  void push(T route) =>
      store.dispatchEvent(event: Event<NavigationState<T>>(reducer: (_) => state..push(route)));

  @override
  void pushReplacement(T route) => store.dispatchEvent(
      event: Event<NavigationState<T>>(reducer: (_) => state..pushReplacement(route)));

  @override
  StateStream<List<T>> get routeStream => state.propertyStream((state) => state.routes);
}
